# Copyright (C) 2016- Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from future import standard_library
standard_library.install_aliases()
from builtins import *  # noqa

from collections import OrderedDict
import logging

from wsgiref.util import setup_testing_defaults

import pytest

from talisker.request_context import request_context
from talisker import logs
logs.configure_test_logging()


@pytest.yield_fixture(autouse=True)
def clean_up_context():
    yield
    request_context.__release_local__()
    logs.StructuredLogger._extra = OrderedDict()
    logs.StructuredLogger._prefix = ''
    logs._logging_configured = False
    logging.getLogger().handlers = []


@pytest.fixture
def environ():
    env = {}
    setup_testing_defaults(env)
    return env


def run_wsgi(app, environ):
    output = {}

    def start_response(status, headers, exc_info=None):
        output['status'] = status
        output['headers'] = headers

    body = app(environ, start_response)

    return body, output['status'], output['headers']



# Copyright (C) 2016- Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from future import standard_library
standard_library.install_aliases()
from builtins import *  # noqa

from werkzeug.local import Local, LocalManager, release_local

# a per request context. Generally, this will be the equivelant of thread local
# storage, but if greenlets are being used, it will be a greenlet local.
request_context = Local()

# used in wsgi stack for clean up
_manager = LocalManager(request_context)
cleanup_middleware = _manager.make_middleware


def cleanup():
    release_local(request_context)



# Copyright (C) 2016- Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from future import standard_library
standard_library.install_aliases()
from builtins import *  # noqa

from .conftest import run_wsgi # noqa

from talisker import wsgi


def app(environ, start_response):
    start_response(200, [])
    return environ


def test_set_environ(environ):
    stack = wsgi.set_environ(app, X=1)
    env, status, headers = run_wsgi(stack, environ)
    assert env['X'] == 1


def test_set_headers(environ):
    stack = wsgi.set_headers(app, {'extra': 'header'})
    env, status, headers = run_wsgi(stack, environ)
    assert ('extra', 'header') in headers


def test_wrapping():
    wrapped = wsgi.wrap(app)

    assert wrapped._talisker_wrapped is True
    assert wrapped._talisker_original_app is app
    assert wrapped is not app

    wrapped2 = wsgi.wrap(wrapped)
    assert wrapped2 is wrapped



# Copyright (C) 2016- Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from future import standard_library
standard_library.install_aliases()
from builtins import *  # noqa

import uuid

from py.test import fixture
import mock

from .conftest import run_wsgi # noqa

from talisker import request_id
from talisker.request_context import request_context


@fixture
def id():
    return str(uuid.uuid4())


def app(environ, start_response):
    start_response(200, [])
    return [
        environ.get('REQUEST_ID'),
        request_id.get(),
        request_context.extra['request_id'],
    ]


REQUEST_ID = 'X-Request-Id'


def test_middleware_with_id(environ, id):
    middleware = request_id.RequestIdMiddleware(app)
    environ[b'HTTP_X_REQUEST_ID'] = id
    body, status, headers = run_wsgi(middleware, environ)
    assert list(set(body)) == [id]
    assert ('X-Request-Id', id) in headers


def test_middleware_without_id(environ, id):
    middleware = request_id.RequestIdMiddleware(app)
    with mock.patch('talisker.request_id.generate') as mock_id:
        mock_id.return_value = id
        body, status, headers = run_wsgi(middleware, environ)
        assert list(set(body)) == [id]
        assert ('X-Request-Id', id) in headers


def test_middleware_alt_header(environ, id):
    middleware = request_id.RequestIdMiddleware(app, 'X-Alternate')
    environ[b'HTTP_X_ALTERNATE'] = id
    body, status, headers = run_wsgi(middleware, environ)
    assert list(set(body)) == [id]
    assert ('X-Alternate', id) in headers


def test_decorator(id):

    @request_id.decorator(lambda: id)
    def test():
        assert request_id.get() == id

    test()
